/*  

    *** Duke Nukem 3D Camera Remover ***
    by Poda

    This will remove all the cameras from a specified map.
    This is useful for watching demos in the map, since you
    won't have to worry about the game switching to a 3rd-person view.
    It basically replaces the sector effector with lotag of 27
    with a blank sprite.  It prints a message stating how many
    cameras were removed.  Expects a version 7 map file.

    Usage:  se27hack mapfile

*/

#include <stdio.h>

//Struct for the map sectors.

typedef struct
{

    short wallptr, wallnum;
    long ceilingz, floorz;
    short ceilingstat, floorstat;
    short ceilingpicnum, ceilingheinum;
    signed char ceilingshade;
    char ceilingpal, ceilingxpanning, ceilingypanning;
    short floorpicnum, floorheinum;
    signed char floorshade;
    char floorpal, floorxpanning, floorypanning;
    char visibility, filler;
    short lotag, hitag, extra;

} sectortype;

sectortype sector[1024];

//Struct for the map walls.

typedef struct
{

    long x, y;
    short point2, nextwall, nextsector, cstat;
    short picnum, overpicnum;
    signed char shade;
    char pal, xrepeat, yrepeat, xpanning, ypanning;
    short lotag, hitag, extra;

} walltype;

walltype wall[8192];

//Struct for the map sprites.

typedef struct
{

   long x, y, z;
   short cstat, picnum;
   signed char shade;
   char pal, clipdist, filler;
   unsigned char xrepeat, yrepeat;
   signed char xoffset, yoffset;
   short sectnum, statnum;
   short ang, owner, xvel, yvel, zvel;
   short lotag, hitag, extra;

} spritetype;

spritetype sprite[4096];

int main(int argc, char **argv)
{

    //Variables for various map information.

    short ang, cursectnum, numsectors, numwalls, numsprites;
    long posx, posy, posz, mapversion;

    int i, count = 0;
    FILE* file;

    //Check for proper command-line usage.

    if (argc < 2)
    {   

        printf("\nUsage:  se27hack mapfile\n");

        return 0;

    }

    //Open the map.

    file = fopen(argv[1], "rb");

    if (file == NULL)
    {

        printf("\nInvalid map!\n");
     
        return 0;

    }

    //Read the map version.  It must be 7L.

    fread(&mapversion, sizeof(mapversion), 1, file);

    if (mapversion != 7L)
    {

        printf("\nInvalid map!\n");
        fclose(file);

        return 0;

    }

    //Read in the remaining map information.

    fread(&posx, sizeof(posx), 1, file);
    fread(&posy, sizeof(posy), 1, file);
    fread(&posz, sizeof(posz), 1, file);
    fread(&ang, sizeof(ang), 1, file);
    fread(&cursectnum, sizeof(cursectnum), 1, file);
    fread(&numsectors, sizeof(numsectors), 1, file);
    fread(sector, sizeof(sectortype), numsectors, file);
    fread(&numwalls, sizeof(numwalls), 1, file);
    fread(wall, sizeof(walltype), numwalls, file);
    fread(&numsprites, sizeof(numsprites), 1, file);
    fread(sprite, sizeof(spritetype), numsprites, file);
    
    fclose(file);

    //Search through all the sprites looking
    //for sector effectors with a lotag of 27.
    //These are the cameras.  Replace them with
    //the first sprite (that ugly square) and
    //make it invisible.

    for (i = 0; i < numsprites; ++i)
        if (sprite[i].lotag == 27 && sprite[i].picnum == 1)
        {   

            sprite[i].picnum = 0;
            sprite[i].cstat |= 0x8000;

            ++count;

        }
    
    //Write the new map.

    file = fopen(argv[1], "wb");

    fwrite(&mapversion, sizeof(mapversion), 1, file);
    fwrite(&posx, sizeof(posx), 1, file);
    fwrite(&posy, sizeof(posy), 1, file);
    fwrite(&posz, sizeof(posz), 1, file);
    fwrite(&ang, sizeof(ang), 1, file);
    fwrite(&cursectnum, sizeof(cursectnum), 1, file);
    fwrite(&numsectors, sizeof(numsectors), 1, file);
    fwrite(sector, sizeof(sectortype), numsectors, file);
    fwrite(&numwalls, sizeof(numwalls), 1, file);
    fwrite(wall, sizeof(walltype), numwalls, file);
    fwrite(&numsprites, sizeof(numsprites), 1, file);
    fwrite(sprite, sizeof(spritetype), numsprites, file);

    fclose(file);

    //Print the number of cameras removed to the user.

    printf("\n%d cameras removed!\n", count);

    return 0;

}
